@@ -1,22 +1,24 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (c) 2012-2018 Pytroll Developers
-
+#
+# Copyright (c) 2012-2020 Pytroll Developers
+#
 # Author(s):
-
+#
 #   Martin Raspaud <martin.raspaud@smhi.se>
-
+#   Janne Kotro <janne.kotro@fmi.fi>
+#   Panu Lahtinen <panu.lahtinen@fmi.fi>
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -46,7 +48,7 @@ from posttroll.message import Message
 from posttroll.publisher import Publish
 from pytroll_collectors.helper_functions import is_uri_on_server
 
-LOGGER = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 JPSS_INSTRUMENTS_FROM_FILENAMES = {"RATMS-RNSCA_": "atms",
                                    "RCRIS-RNSCA_": "cris",
@@ -65,11 +67,10 @@ SCISYS_NAMES = {'Suomi-NPP': 'NPP',
 
 
 class TwoMetMessage(object):
-
-    """Interperter for 2met! messages.
-    """
+    """Interperter for 2met! messages."""
 
     def __init__(self, mstring=None):
+        """Init the message."""
         self._id = 0
         self._type = ""
         self._time = datetime.utcnow()
@@ -79,8 +80,7 @@ class TwoMetMessage(object):
         self._attrs = {}
 
     def _internal_decode(self, mstring):
-        """Decode 2met! messages, internal format.
-        """
+        """Decode 2met! messages, internal format."""
         dummy, content = mstring.split("[", 1)
         content = content.rsplit("]", 1)[0]
         dic = dict((item.split("=", 1) for item in content.split(", ", 3)))
@@ -94,8 +94,7 @@ class TwoMetMessage(object):
         self._type = eval(dic["type"])
 
     def _xml_decode(self, mstring):
-        """Decode xml 2met! messages.
-        """
+        """Decode xml 2met! messages."""
         root = etree.fromstring(mstring)
         self._attrs = dict(root.items())
 
@@ -108,37 +107,37 @@ class TwoMetMessage(object):
                 self.body = child.text
 
     def _decode(self, mstring):
-        """Decode 2met! messages.
-        """
-
+        """Decode 2met! messages."""
         if mstring.startswith("Message["):
             self._internal_decode(mstring)
         elif mstring.startswith("<message"):
             try:
                 self._xml_decode(mstring)
-            except:
-                LOGGER.exception("Spurious message! %s", str(mstring))
+            except Exception:
+                logger.exception("Spurious message! %s", str(mstring))
         else:
-            LOGGER.warning("Don't know how to decode message: %s",
+            logger.warning("Don't know how to decode message: %s",
                            str(mstring))
 
 
 def pass_name(utctime, satellite):
-    """Construct a unique pass name from a risetime and a satellite name.
-    """
+    """Construct a unique pass name from a risetime and a satellite name."""
     # return utctime.strftime("%Y%m%dT%H%M%S") + "_".join(satellite.split(" "))
     return utctime, "_".join(satellite.split(" "))
 
 
 class PassRecorder(dict):
+    """Recorder for passes."""
 
     def iter(self):
+        """Return an iterator over the passes."""
         if hasattr(self, 'iteritems'):
-            return self.iteritems()
+            return self.iteritems()  # noqa
         else:
             return self.items()
 
     def get(self, key, default=None):
+        """Get a pass."""
         utctime, satellite = key
         for (rectime, recsat), val in self.iter():
             if(recsat == satellite and
@@ -149,11 +148,11 @@ class PassRecorder(dict):
 
 
 class MessageReceiver(object):
+    """Interprets received messages between stop reception and file dispatch."""
 
-    """Interprets received messages between stop reception and file dispatch.
-    """
-
-    def __init__(self, emitter, excluded_satellite_list=None):
+    def __init__(self, emitter, excluded_satellite_list=None,
+                 target_server=None, ftp_prefix=None):
+        """Init the receiver."""
         self._received_passes = PassRecorder()
         self._distributed_files = {}
         self._emitter = emitter
@@ -161,12 +160,13 @@ class MessageReceiver(object):
             self._excluded_platforms = excluded_satellite_list
         else:
             self._excluded_platforms = []
+        self._target_server = target_server
+        self._ftp_prefix = ftp_prefix
 
     def add_pass(self, message):
-        """Formats pass info and adds it to the object.
-        """
+        """Format pass info and add it to the object."""
         info = dict((item.split(": ", 1) for item in message.split(", ", 3)))
-        LOGGER.info("Adding pass: %s", str(info))
+        logger.info("Adding pass: %s", str(info))
         pass_info = {}
         for key, val in info.items():
             pass_info[key.lower()] = val
@@ -181,14 +181,13 @@ class MessageReceiver(object):
             pass_info['orbit_number'] = int(pass_info['orbit number'])
             del pass_info['orbit number']
         else:
-            LOGGER.warning("No 'orbit number' in message!")
+            logger.warning("No 'orbit number' in message!")
 
         pname = pass_name(pass_info["start_time"], pass_info["satellite"])
         self._received_passes[pname] = pass_info
 
     def clean_passes(self, days=1):
-        """Clean old passes from the pass dict (_received_passes).
-        """
+        """Clean old passes from the pass dict (`_received_passes`)."""
         oldies = []
 
         for key, val in self._received_passes.items():
@@ -202,7 +201,7 @@ class MessageReceiver(object):
         """React to a file dispatch message."""
         url = urlsplit(url)
         dummy, filename = os.path.split(url.path)
-        LOGGER.debug("filename = %s", filename)
+        logger.debug("filename = %s", filename)
         # TODO: Should not make any assumptions on filename formats, should
         # load a description of it from a config file instead.
         if filename.endswith(".hmf"):
@@ -246,7 +245,7 @@ class MessageReceiver(object):
                     "Unrecognized satellite ID: " + pds["apid1"][:3])
 
             if not satellite or satellite in self._excluded_platforms:
-                LOGGER.debug("Platform name %s is excluded...", str(satellite))
+                logger.debug("Platform name %s is excluded...", str(satellite))
                 return None
 
             swath = self._received_passes.get(pname, {}).copy()
@@ -295,14 +294,14 @@ class MessageReceiver(object):
                     break
 
             if not file_ok:
-                LOGGER.warning("Seems to be a NPP/JPSS RDR "
+                logger.warning("Seems to be a NPP/JPSS RDR "
                                "file but name is not standard!")
-                LOGGER.warning("filename = %s", filename)
+                logger.warning("filename = %s", filename)
                 return None
 
             # satellite = "Suomi-NPP, NOAA-20, NOAA-21,..."
             if not satellite or satellite in self._excluded_platforms:
-                LOGGER.debug("Platform name %s is excluded...", str(satellite))
+                logger.debug("Platform name %s is excluded...", str(satellite))
                 return None
 
             mda["start_time"] = \
@@ -356,7 +355,7 @@ class MessageReceiver(object):
             falltime = datetime.strptime(filename[32:47], "%Y%m%d%H%M%SZ")
 
             pname = pass_name(risetime, satellite.upper())
-            LOGGER.debug("pname= % s", str(pname))
+            logger.debug("pname= % s", str(pname))
             swath = self._received_passes.get(pname, {}).copy()
             swath.pop('satellite', None)
             swath["start_time"] = risetime
@@ -386,7 +385,7 @@ class MessageReceiver(object):
                                          url.query,
                                          url.fragment))
         else:
-            LOGGER.debug("url.scheme not expected: %s", url.scheme)
+            logger.debug("url.scheme not expected: %s", url.scheme)
 
         swath["uid"] = os.path.split(url.path)[1]
         swath["uri"] = uri
@@ -410,14 +409,23 @@ class MessageReceiver(object):
             if is_uri_on_server(url, strict=True):
                 return self.handle_distrib(url)
         elif message.body.startswith(dispatch_prefix2):
-            filename, arr, url = message.body[len(dispatch_prefix2):].split(" ")
-            url = 'ftp://' + url.replace(':', '')
-            del arr
-            LOGGER.debug("filename = <%s> url = <%s>", filename, url)
-            url = compose_dest_url(filename, url)
-            LOGGER.debug("url = %s", url)
-            if is_uri_on_server(url, strict=True):
+            filename, _, url = message.body[len(dispatch_prefix2):].split(" ")
+            ip_address, path = url.split(":")
+            logger.debug("Sent to IP: %s", str(ip_address))
+            logger.debug("Target server IPs: %s ", str(self._target_server))
+            if ip_address in self._target_server:
+                logger.debug("Found correct target server: %s", ip_address)
+                if self._ftp_prefix is not None:
+                    path = os.path.join(self._ftp_prefix, path)
+                    logger.debug("ftp_prefix added: %s", path)
+                url = 'ftp://' + ip_address + path
+                logger.debug("filename = <%s> url = <%s>", filename, url)
+                url = compose_dest_url(filename, url)
+                logger.debug("url = %s", url)
                 return self.handle_distrib(url)
+            else:
+                logger.debug("File is not on targer server.")
+                return None
 
 
 def compose_dest_url(source, dest):
@@ -432,8 +440,10 @@ def compose_dest_url(source, dest):
 
 
 class GMCSubscriber(object):
+    """Listener for GMC messages."""
 
     def __init__(self, host, port):
+        """Init the subscriber."""
         self._host = host
         self._port = port
         self._sock = None
@@ -442,14 +452,13 @@ class GMCSubscriber(object):
         self.loop = True
 
     def recv(self):
-        """Receive messages.
-        """
+        """Receive messages."""
         while self.loop:
             self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             try:
                 self._sock.connect((self._host, self._port))
             except socket.error:
-                LOGGER.error("Cannot connect to %s, retrying in 60 seconds.",
+                logger.error("Cannot connect to %s, retrying in 60 seconds.",
                              str((self._host, self._port)))
                 sleep(60)
                 continue
@@ -463,7 +472,7 @@ class GMCSubscriber(object):
                     else:
                         if not data:
                             break
-                        self.msg += data
+                        self.msg += data.decode('utf-8')
                         messages = self.msg.split("</message>")
                         if len(messages) > 1:
                             for mess in messages[:-1]:
@@ -480,44 +489,51 @@ class GMCSubscriber(object):
                 self._sock.close()
 
     def stop(self):
+        """Stop the listener."""
         self.loop = False
 
 
-def receive_from_zmq(host, port, station, environment, excluded_platforms, days=1):
-    """Receive 2met! messages from zeromq.
-    """
+def receive_from_zmq(host, port, station, environment, excluded_platforms,
+                     target_server, ftp_prefix, topic_postfix,
+                     publish_port=0, nameservers=None, days=1):
+    """Receive 2met! messages from zeromq."""
+    logger.debug("target_server: %s", str(target_server))
+    logger.debug("ftp_prefix: %s", str(ftp_prefix))
+    logger.debug("topic_postfix: %s", str(topic_postfix))
+    logger.debug("station %s", str(station))
+    logger.debug(type(target_server))
 
     # socket = Subscriber(["tcp://localhost:9331"], ["2met!"])
     sock = GMCSubscriber(host, port)
-    msg_rec = MessageReceiver(host, excluded_platforms)
+    msg_rec = MessageReceiver(host, excluded_platforms,
+                              target_server, ftp_prefix)
 
-    with Publish("receiver", 0, ["HRPT 0", "PDS", "RDR", "EPS 0"]) as pub:
+    with Publish("receiver", port=publish_port,
+                 aliases=["HRPT 0", "PDS", "RDR", "EPS 0"],
+                 nameservers=nameservers) as pub:
         for rawmsg in sock.recv():
             # TODO:
             # - Watch for idle time in order to detect a hangout
-            LOGGER.debug("receive from 2met! %s", str(rawmsg))
+            logger.debug("receive from 2met! %s", str(rawmsg))
             string = TwoMetMessage(rawmsg)
             to_send = msg_rec.receive(string)
+            logger.debug("to_send: %s", str(to_send))
             if to_send is None:
                 continue
-            subject = "/".join(("", to_send['format'],
-                                to_send['data_processing_level'],
-                                station, environment,
-                                "polar", "direct_readout"))
+            if topic_postfix is not None:
+                subject = "/".join(("", to_send['format'],
+                                   to_send['data_processing_level'],
+                                   topic_postfix))
+            else:
+                subject = "/".join(("", to_send['format'],
+                                   to_send['data_processing_level'],
+                                   station, environment,
+                                   "polar", "direct_readout"))
+            logger.debug("Subject: %s", str(subject))
             msg = Message(subject,
                           "file",
                           to_send).encode()
-            LOGGER.debug("publishing %s", str(msg))
+            logger.debug("publishing %s", str(msg))
             pub.send(msg)
             if days:
                 msg_rec.clean_passes(days)
-
-
-if __name__ == '__main__':
-    rawmsg = '<message timestamp="2018-02-07T02:06:05" sequence="152" severity="INFO" messageID="0" type="2met.dispat.suctrn.info" sourcePU="MERLIN" sourceSU="Dispatch" sourceModule="DISPAT" sourceInstance="1"><body>SUCTRN RATMS-RNSCA_npp_d20180207_t0205166_e0205486_b00001_c20180207020559052000_all-_dev.h5 -&gt; 10.120.1.66:/tmp</body></message>'
-    rawmsg = '<message timestamp="2018-02-12T11:14:06" sequence="9194" severity="INFO" messageID="0" type="2met.dispat.suctrn.info" sourcePU="MERLIN" sourceSU="Dispatch" sourceModule="DISPAT" sourceInstance="1"><body>SUCTRN 20180212110043_NOAA_19.hmf -&gt; 192.168.122.1:/tmp</body></message>'
-
-    string = TwoMetMessage(rawmsg)
-    msg_rec = MessageReceiver('merlin')
-    ret = msg_rec.receive(string)
-    print(ret)
