@@ -20,22 +20,22 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""Gather GEO stationary segments, or polar satellite granules for one
-timestep, and send them in a bunch as a dataset.
+"""Gather segments.
+
+Gather GEO stationary segments, or polar satellite granules for one timestep,
+and send them in a bunch as a dataset.
 """
 
 import datetime as dt
 import logging
 import logging.handlers
-import os.path
-from six.moves.queue import Empty as queue_empty
-import time
+from six.moves.queue import Empty
 from collections import OrderedDict
-from six.moves.urllib.parse import urlparse, urlunparse
+from six.moves.urllib.parse import urlparse
 
 from posttroll import message, publisher
 from posttroll.listener import ListenerContainer
-from trollsift import Parser, compose
+from trollsift import Parser
 
 SLOT_NOT_READY = 0
 SLOT_NONCRITICAL_NOT_READY = 1
@@ -48,14 +48,13 @@ REMOVE_TAGS = {'path', 'segment'}
 
 
 class SegmentGatherer(object):
-
-    """Gatherer for geostationary satellite segments and multifile polar
-    satellite granules."""
+    """Gatherer for geostationary satellite segments and multifile polar satellite granules."""
 
     _listener = None
     _publisher = None
 
     def __init__(self, config):
+        """Initialize the segment gatherer."""
         self._config = config
         self._subject = None
         self._patterns = config['patterns']
@@ -72,9 +71,42 @@ class SegmentGatherer(object):
                          key in self._patterns}
 
         self.time_name = config.get('time_name', 'start_time')
+        # Floor the scene start time to the given full minutes
+        self._group_by_minutes = config.get('group_by_minutes', None)
+
+        # This get the 'keep_parsed_keys' valid for all patterns
+        self._keep_parsed_keys = config.get('keep_parsed_keys', [])
 
         self.logger = logging.getLogger("segment_gatherer")
         self._loop = False
+        self._providing_server = config.get('providing_server')
+
+        # Convert check time into int minutes variables
+        for key in self._patterns:
+            if "start_time_pattern" in self._patterns[key]:
+                time_conf = self._patterns[key]["start_time_pattern"]
+                start_time_str = time_conf.get("start_time", "00:00")
+                end_time_str = time_conf.get("end_time", "23:59")
+                delta_time_str = time_conf.get("delta_time", "00:01")
+
+                start_h, start_m = start_time_str.split(':')
+                end_h, end_m = end_time_str.split(':')
+                delta_h, delta_m = delta_time_str.split(':')
+                interval = {}
+                interval["start"] = (60 * int(start_h)) + int(start_m)
+                interval["end"] = (60 * int(end_h)) + int(end_m)
+                interval["delta"] = (60 * int(delta_h)) + int(delta_m)
+
+                # Start-End time across midnight
+                interval["midnight"] = False
+                if interval["start"] > interval["end"]:
+                    interval["end"] += 24 * 60
+                    interval["midnight"] = True
+                self._patterns[key]["_start_time_pattern"] = interval
+                self.logger.info("Start Time pattern '%s' " +
+                                 "filter start:%s end:%s delta:%s",
+                                 key, start_time_str, end_time_str,
+                                 delta_time_str)
 
     def _clear_data(self, time_slot):
         """Clear data."""
@@ -82,7 +114,7 @@ class SegmentGatherer(object):
             del self.slots[time_slot]
 
     def _init_data(self, mda):
-        """Init wanted, all and critical files"""
+        """Init wanted, all and critical files."""
         # Init metadata struct
         metadata = mda.copy()
 
@@ -145,8 +177,9 @@ class SegmentGatherer(object):
 
     def _compose_filenames(self, key, time_slot, itm_str):
         """Compose filename set()s based on a pattern and item string.
-        itm_str is formated like ':PRO,:EPI' or 'VIS006:8,VIS008:1-8,...'"""
 
+        itm_str is formated like ':PRO,:EPI' or 'VIS006:8,VIS008:1-8,...'
+        """
         # Empty set
         result = set()
 
@@ -163,8 +196,18 @@ class SegmentGatherer(object):
         meta = _copy_without_ignore_items(meta,
                                           ignored_keys=var_tags)
 
+        parser = self._parsers[key]
+
         for itm in itm_str.split(','):
             channel_name, segments = itm.split(':')
+            if channel_name == '' and segments == '':
+                # If the filename pattern has no segments/channels,
+                # add the "plain" globified filename to the filename
+                # set
+                if ('channel_name' not in parser.fmt and
+                        'segment' not in parser.fmt):
+                    result.add(parser.globify(meta))
+                continue
             segments = segments.split('-')
             if len(segments) > 1:
                 format_string = '%d'
@@ -176,7 +219,7 @@ class SegmentGatherer(object):
             meta['channel_name'] = channel_name
             for seg in segments:
                 meta['segment'] = seg
-                fname = self._parsers[key].globify(meta)
+                fname = parser.globify(meta)
 
                 result.add(fname)
 
@@ -184,7 +227,6 @@ class SegmentGatherer(object):
 
     def _publish(self, time_slot, missing_files_check=True):
         """Publish file dataset and reinitialize gatherer."""
-
         data = self.slots[time_slot]
 
         # Diagnostic logging about delayed ...
@@ -229,6 +271,7 @@ class SegmentGatherer(object):
         self.logger = logger
 
     def update_timeout(self, time_slot):
+        """Update the timeout."""
         timeout = dt.datetime.utcnow() + self._timeliness
         self.slots[time_slot]['timeout'] = timeout
         self.logger.info("Setting timeout to %s for slot %s.",
@@ -270,12 +313,11 @@ class SegmentGatherer(object):
         return self.get_collection_status(status, slot['timeout'], time_slot)
 
     def get_collection_status(self, status, timeout, time_slot):
-        """Determine the overall status of the collection"""
+        """Determine the overall status of the collection."""
         if len(status) == 0:
             return SLOT_NOT_READY
 
         status_values = list(status.values())
-
         if all([val == SLOT_READY for val in status_values]):
             self.logger.info("Required files received "
                              "for slot %s.", time_slot)
@@ -291,7 +333,10 @@ class SegmentGatherer(object):
                 return SLOT_READY
             elif all([val == SLOT_NONCRITICAL_NOT_READY for val in
                       status_values]):
-                return SLOT_READY
+                for key in status.keys():
+                    if len(self.slots[time_slot][key]['received_files']) > 0:
+                        return SLOT_READY
+                return SLOT_OBSOLETE_TIMEOUT
             else:
                 self.logger.warning("Timeout occured and required files "
                                     "were not present, data discarded for "
@@ -306,21 +351,31 @@ class SegmentGatherer(object):
         if SLOT_READY_BUT_WAIT_FOR_MORE in status_values:
             return SLOT_READY_BUT_WAIT_FOR_MORE
 
+    def _generate_publish_service_name(self):
+        publish_service_name = "segment_gatherer"
+        for key in sorted(self._patterns):
+            publish_service_name += "_" + str(key)
+        return publish_service_name
+
     def _setup_messaging(self):
-        """Setup messaging"""
+        """Set up messaging."""
         self._subject = self._config['posttroll']['publish_topic']
         topics = self._config['posttroll'].get('topics')
         addresses = self._config['posttroll'].get('addresses')
         publish_port = self._config['posttroll'].get('publish_port', 0)
         nameservers = self._config['posttroll'].get('nameservers', [])
-        self._listener = ListenerContainer(topics=topics, addresses=addresses)
-        self._publisher = publisher.NoisyPublisher("segment_gatherer",
+        services = self._config['posttroll'].get('services')
+        self._listener = ListenerContainer(topics=topics, addresses=addresses, services=services)
+        # Name each segment_gatherer with the section/patterns name.
+        # This way the user can subscribe to a specific segment_gatherer service instead of all.
+        publish_service_name = self._generate_publish_service_name()
+        self._publisher = publisher.NoisyPublisher(publish_service_name,
                                                    port=publish_port,
                                                    nameservers=nameservers)
         self._publisher.start()
 
     def run(self):
-        """Run SegmentGatherer"""
+        """Run SegmentGatherer."""
         self._setup_messaging()
 
         self._loop = True
@@ -353,10 +408,13 @@ class SegmentGatherer(object):
             except KeyboardInterrupt:
                 self.stop()
                 continue
-            except queue_empty:
+            except Empty:
                 continue
 
             if msg.type == "file":
+                # If providing server is configured skip message if not from providing server
+                if self._providing_server and self._providing_server != msg.host:
+                    continue
                 self.logger.info("New message received: %s", str(msg))
                 self.process(msg)
 
@@ -371,10 +429,14 @@ class SegmentGatherer(object):
             self._publisher.stop()
 
     def process(self, msg):
-        """Process message"""
+        """Process message."""
         mda = None
 
-        uid = msg.data['uid']
+        try:
+            uid = msg.data['uid']
+        except KeyError:
+            self.logger.debug("Ignoring: %s", str(msg))
+            return
 
         # Find the correct parser for this file
         key = self.key_from_fname(uid)
@@ -384,10 +446,28 @@ class SegmentGatherer(object):
 
         parser = self._parsers[key]
         mda = parser.parse(msg.data["uid"])
+        mda = self._floor_time(mda, key)
 
-        metadata = copy_metadata(mda, msg)
+        # Check if each pattern contains a seperate 'keep_parsed_keys'
+        local_keep_parsed_keys = self._patterns[key].get('keep_parsed_keys', [])
+        metadata = copy_metadata(mda, msg,
+                                 keep_parsed_keys=self._keep_parsed_keys,
+                                 local_keep_parsed_keys=local_keep_parsed_keys)
 
-        time_slot = self._find_time_slot(metadata["start_time"])
+        # Check if time of the raw is in scheduled range
+        if "_start_time_pattern" in self._patterns[key]:
+            schedule_ok = self.check_schedule_time(
+                self._patterns[key]["_start_time_pattern"],
+                metadata[self.time_name])
+            if not schedule_ok:
+                self.logger.info("Hour pattern '%s' skip: %s" +
+                                 " for start_time: %s:%s",
+                                 key, msg.data["uid"],
+                                 metadata[self.time_name].hour,
+                                 metadata[self.time_name].minute)
+                return
+
+        time_slot = self._find_time_slot(metadata[self.time_name])
 
         # Init metadata etc if this is the first file
         if time_slot not in self.slots:
@@ -396,8 +476,28 @@ class SegmentGatherer(object):
         # Check if this file has been received already
         self.add_file(time_slot, key, mda, msg.data)
 
+    def _floor_time(self, mda, key=None):
+        """Floor time to full minutes."""
+        # This is the 'group_by_minutes' for all patterns
+        group_by_minutes = self._group_by_minutes
+
+        # Check if 'group_by_minutes' is given in the \key\ pattern
+        if key is not None and 'group_by_minutes' in self._patterns[key]:
+            group_by_minutes = self._patterns[key]['group_by_minutes']
+        elif self._group_by_minutes is None:
+            return mda
+
+        start_time = mda[self.time_name]
+        mins = start_time.minute
+        fl_mins = int(mins / group_by_minutes) * group_by_minutes
+        start_time = dt.datetime(start_time.year, start_time.month,
+                                 start_time.day, start_time.hour, fl_mins, 0)
+        mda[self.time_name] = start_time
+
+        return mda
+
     def add_file(self, time_slot, key, mda, msg_data):
-        """Add file to the correct filelist"""
+        """Add file to the correct filelist."""
         uri = urlparse(msg_data['uri']).path
         uid = msg_data['uid']
         slot = self.slots[time_slot][key]
@@ -452,7 +552,7 @@ class SegmentGatherer(object):
         self.logger.info("%s processed", uid)
 
     def key_from_fname(self, uid):
-        """"""
+        """Get the keys from a filename."""
         for key in self._parsers:
             try:
                 _ = self._parsers[key].parse(uid)
@@ -461,8 +561,10 @@ class SegmentGatherer(object):
                 pass
 
     def _find_time_slot(self, time_obj):
-        """Find time slot and return the slot as a string.  If no slots are
-        close enough, return *str(time_obj)*"""
+        """Find time slot and return the slot as a string.
+
+        If no slots are close enough, return *str(time_obj)*
+        """
         for slot in self.slots:
             time_slot = self.slots[slot]['metadata'][self.time_name]
             time_diff = time_obj - time_slot
@@ -472,12 +574,26 @@ class SegmentGatherer(object):
 
         return str(time_obj)
 
+    def check_schedule_time(self, check_time, raw_start_time):
+        """Check if raw time is inside configured interval."""
+        time_ok = False
+
+        # Convert check time into int variables
+        raw_time = (60 * raw_start_time.hour) + raw_start_time.minute
+        if check_time["midnight"] and raw_time < check_time["start"]:
+            raw_time += 24 * 60
+
+        # Check start and end time
+        if raw_time >= check_time["start"] and raw_time <= check_time["end"]:
+            # Raw time in range, check interval
+            if ((raw_time - check_time["start"]) % check_time["delta"]) == 0:
+                time_ok = True
+
+        return time_ok
+
 
 def _copy_without_ignore_items(the_dict, ignored_keys='ignore'):
-    """
-    get a copy of *the_dict* without entries having substring
-    'ignore' in key
-    """
+    """Get a copy of *the_dict* without entries having substring 'ignore' in key."""
     if not isinstance(ignored_keys, (list, tuple, set)):
         ignored_keys = [ignored_keys, ]
     new_dict = {}
@@ -499,7 +615,7 @@ def ini_to_dict(fname, section):
     posttroll = conf['posttroll']
     posttroll['topics'] = config.get(section, 'topics').split()
     try:
-        nameservers = config.get(section, 'nameserver')
+        nameservers = config.get(section, 'nameservers')
         nameservers = nameservers.split()
     except (NoOptionError, ValueError):
         nameservers = None
@@ -512,6 +628,13 @@ def ini_to_dict(fname, section):
         addresses = None
     posttroll['addresses'] = addresses
 
+    try:
+        services = config.get(section, 'services')
+        services = services.split()
+    except (NoOptionError, ValueError):
+        services = ""
+    posttroll['services'] = services
+
     try:
         publish_port = config.get(section, 'publish_port')
     except NoOptionError:
@@ -549,11 +672,31 @@ def ini_to_dict(fname, section):
     except (NoOptionError, ValueError):
         conf['num_files_premature_publish'] = -1
 
+    try:
+        conf['group_by_minutes'] = config.getint(section, 'group_by_minutes')
+    except (NoOptionError, ValueError):
+        pass
+
+    try:
+        kps = config.get(section, 'keep_parsed_keys')
+        conf['keep_parsed_keys'] = kps.split()
+    except NoOptionError:
+        pass
+
+    try:
+        conf['providing_server'] = config.get(section, "providing_server")
+    except (NoOptionError, ValueError):
+        conf['providing_server'] = None
+
     return conf
 
 
-def copy_metadata(mda, msg):
-    """Copy metada from filename and message to a combined dictionary"""
+def copy_metadata(mda, msg, keep_parsed_keys=None, local_keep_parsed_keys=None):
+    """Copy metada from filename and message to a combined dictionary."""
+    if keep_parsed_keys is None:
+        keep_parsed_keys = []
+    if local_keep_parsed_keys is None:
+        local_keep_parsed_keys = []
     metadata = {}
     # Use values parsed from the filename as basis
     for key in mda:
@@ -562,7 +705,8 @@ def copy_metadata(mda, msg):
 
     # Update with data given in the message
     for key in msg.data:
-        if key not in DO_NOT_COPY_KEYS:
+        # If time name is given, do not overwrite it
+        if key not in DO_NOT_COPY_KEYS and key not in keep_parsed_keys and key not in local_keep_parsed_keys:
             metadata[key] = msg.data[key]
 
     return metadata
